"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_hooks_1 = require("@testing-library/react-hooks");
const _1 = require(".");
describe('Data loader hook', () => {
    test('should allow any call fn', () => {
        () => (0, _1.useAsyncRequest)({ callFn: () => __awaiter(void 0, void 0, void 0, function* () { }) });
        () => (0, _1.useAsyncRequest)({ callFn: (a, b, c) => __awaiter(void 0, void 0, void 0, function* () { }) });
    });
    test('should delay data loading till reload is called', () => __awaiter(void 0, void 0, void 0, function* () {
        const demoData = 'hello';
        const args = ['products', 'filter'];
        const mockLoadFn = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { return demoData; }));
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, _1.useAsyncRequest)({ args, callOnInit: false, callFn: mockLoadFn }));
        expect(result.current.response).toEqual(null);
        expect(result.current.loading).toEqual(false);
        expect(result.current.hasLoaded).toEqual(false);
        expect(result.current.error).toEqual(undefined);
        expect(mockLoadFn).toHaveBeenCalledTimes(0);
        (0, react_hooks_1.act)(() => {
            result.current.call();
        });
        expect(mockLoadFn).toHaveBeenCalledTimes(1);
        expect(mockLoadFn).toHaveBeenCalledWith('products', 'filter');
        expect(result.current.loading).toEqual(true);
        expect(result.current.hasLoaded).toEqual(false);
        expect(result.current.response).toEqual(null);
        yield waitForNextUpdate();
        expect(result.current.loading).toEqual(false);
        expect(result.current.hasLoaded).toEqual(true);
        expect(result.current.response).toEqual(demoData);
        expect(result.current.error).toEqual(undefined);
    }));
    test('should load data immediately', () => __awaiter(void 0, void 0, void 0, function* () {
        const demoData = 'hello';
        const args = ['products', 'filter'];
        const mockLoadFn = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { return demoData; }));
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, _1.useAsyncRequest)({ args, callOnInit: true, callFn: mockLoadFn }));
        expect(result.current.loading).toEqual(true);
        expect(mockLoadFn).toHaveBeenCalledTimes(1);
        expect(mockLoadFn).toHaveBeenCalledWith('products', 'filter');
        yield waitForNextUpdate();
        expect(result.current.loading).toEqual(false);
        expect(result.current.hasLoaded).toEqual(true);
        expect(result.current.response).toEqual(demoData);
        expect(result.current.error).toEqual(undefined);
    }));
    test('should allow args override', () => __awaiter(void 0, void 0, void 0, function* () {
        const demoData = 'hello';
        const args = ['products', 'filter'];
        const mockLoadFn = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { return demoData; }));
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, _1.useAsyncRequest)({ args, callOnInit: false, callFn: mockLoadFn }));
        expect(result.current.response).toEqual(null);
        expect(result.current.loading).toEqual(false);
        expect(result.current.hasLoaded).toEqual(false);
        expect(result.current.error).toEqual(undefined);
        (0, react_hooks_1.act)(() => {
            result.current.reload(['hello']);
        });
        expect(result.current.loading).toEqual(true);
        expect(result.current.hasLoaded).toEqual(false);
        expect(mockLoadFn).toHaveBeenCalledWith('hello');
        yield waitForNextUpdate();
        expect(result.current.loading).toEqual(false);
        expect(result.current.hasLoaded).toEqual(true);
        expect(result.current.response).toEqual(demoData);
        expect(result.current.error).toEqual(undefined);
    }));
    test('should handle multiple async calls', () => __awaiter(void 0, void 0, void 0, function* () {
        const args = ['products', 'filter'];
        const demoData = ['call 0 reponse', 'call 1 reponse', 'call 2 reponse'];
        // const r = [];
        const mockLoadFn = jest.fn().mockImplementation((i) => __awaiter(void 0, void 0, void 0, function* () { return demoData[i]; }));
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, _1.useAsyncRequest)({ args, callOnInit: false, callFn: mockLoadFn }));
        (0, react_hooks_1.act)(() => {
            result.current.reload([0]);
        });
        (0, react_hooks_1.act)(() => {
            result.current.reload([1]);
        });
        (0, react_hooks_1.act)(() => {
            result.current.reload([2]);
        });
        expect(result.current.loading).toEqual(true);
        yield waitForNextUpdate();
        expect(mockLoadFn).toHaveBeenCalledTimes(3);
        expect(mockLoadFn.mock.calls[0]).toEqual([0]);
        expect(result.current.loading).toEqual(false);
        expect(result.current.hasLoaded).toEqual(true);
        expect(result.current.response).toEqual(demoData[2]);
        expect(result.current.error).toEqual(undefined);
    }));
    test('should handle multiple async calls and only return last called', () => __awaiter(void 0, void 0, void 0, function* () {
        const args = ['products', 'filter'];
        const demoData = ['call 0 reponse', 'call 1 reponse', 'call 2 reponse'];
        const r = [];
        const delayedPromise = () => new Promise((res) => {
            r.push(res);
        });
        const mockLoadFn = jest.fn().mockImplementation((i) => __awaiter(void 0, void 0, void 0, function* () {
            yield delayedPromise();
            return demoData[i];
        }));
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, _1.useAsyncRequest)({ args, callOnInit: false, callFn: mockLoadFn }));
        /* We call reload 3 times with 3 different values */
        (0, react_hooks_1.act)(() => {
            result.current.reload([0]);
        });
        (0, react_hooks_1.act)(() => {
            result.current.reload([1]);
        });
        (0, react_hooks_1.act)(() => {
            result.current.reload([2]);
        });
        // await waitForNextUpdate();
        expect(result.current.loading).toEqual(true);
        expect(mockLoadFn).toHaveBeenCalledTimes(3);
        expect(mockLoadFn.mock.calls[0]).toEqual([0]);
        expect(result.current.loading).toEqual(true);
        /* Test that returning the first async call does not set loading to false
        as it is still waiting for the final call */
        (0, react_hooks_1.act)(() => {
            r[0]();
        });
        yield waitForNextUpdate();
        expect(result.current.loading).toEqual(true);
        expect(result.current.response).toEqual(null);
        /* Test that returning the third async call sets loading to false and returns the results */
        (0, react_hooks_1.act)(() => {
            r[2]();
        });
        yield waitForNextUpdate();
        expect(result.current.response).toEqual(demoData[2]);
        expect(result.current.loading).toEqual(false);
        /* Test that returning the second after the third does not change the state */
        (0, react_hooks_1.act)(() => {
            r[1]();
        });
        // // await waitForNextUpdate();
        expect(result.current.response).toEqual(demoData[2]);
        expect(result.current.loading).toEqual(false);
        // /* Reloading the function clears everything back to the loading state */
        (0, react_hooks_1.act)(() => {
            result.current.reload([0]);
        });
        yield waitForNextUpdate();
        expect(result.current.loading).toEqual(true);
        expect(result.current.response).toEqual(null);
    }));
    test('should return error message if failed', () => __awaiter(void 0, void 0, void 0, function* () {
        const args = ['products', 'filter'];
        const mockLoadFn = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () {
            throw Error();
        }));
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, _1.useAsyncRequest)({ args, callOnInit: true, callFn: mockLoadFn }));
        expect(result.current.loading).toEqual(true);
        expect(mockLoadFn).toHaveBeenCalledWith('products', 'filter');
        yield waitForNextUpdate();
        expect(result.current.loading).toEqual(false);
        expect(result.current.hasLoaded).toEqual(true);
        expect(result.current.response).toEqual(null);
        expect(result.current.error).toEqual(new Error('Failed to load'));
    }));
    test('should update call count', () => __awaiter(void 0, void 0, void 0, function* () {
        const demoData = 'hello';
        const args = ['products', 'filter'];
        const mockLoadFn = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { return demoData; }));
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, _1.useAsyncRequest)({ args, callOnInit: true, callFn: mockLoadFn }));
        expect(result.current.callCount).toEqual(0);
        expect(result.current.loading).toEqual(true);
        expect(mockLoadFn).toHaveBeenCalledTimes(1);
        expect(mockLoadFn).toHaveBeenCalledWith('products', 'filter');
        yield waitForNextUpdate();
        expect(result.current.hasLoaded).toEqual(true);
        expect(result.current.response).toEqual(demoData);
        expect(result.current.error).toEqual(undefined);
        expect(result.current.callCount).toEqual(1);
        (0, react_hooks_1.act)(() => {
            result.current.reload([0]);
        });
        yield waitForNextUpdate();
        expect(result.current.callCount).toEqual(2);
    }));
    test('should call callback on load complete', () => __awaiter(void 0, void 0, void 0, function* () {
        const demoData = 'hello';
        const args = ['products', 'filter'];
        const mockLoadFn = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { return demoData; }));
        const callback = jest.fn();
        const { waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, _1.useAsyncRequest)({
            args,
            callOnInit: true,
            callFn: mockLoadFn,
            callback,
        }));
        expect(callback).not.toHaveBeenCalledWith(demoData);
        yield waitForNextUpdate();
        expect(callback).toHaveBeenCalledWith(demoData, args);
    }));
});
//# sourceMappingURL=use-async-request.test.js.map